unit Enemy;

interface

uses eXgine, SysUtils, Math, MyInterface, EngineUtils, Consts, SpriteEngine, OpenGL;

const EnemyWind      = 1;
      EnemyTree      = 2;
      EnemySold      = 3;
      EnemyMaxCount  = 3;
      EnemyCount     = 8;
      EnemyW         = 64;
      EnemyH         = 64;
      FrameSpeed     = 0.15;
      EnemyMaxReload = 60;
      EnemySoldHJump = 96;
      WarningDist    = 30;
      TmpSpeed       = 2;

type TEnemyParam = record
       Health    : integer;
       Score     : integer;
       Speed     : integer;
       Vector    : integer;
       TimeReload: integer;
       RadiusAgr : integer;
       Jump      : boolean;
       HeightJump: integer;
     end;

var EnemyParam: array[1..EnemyMaxCount] of TEnemyParam =
    (
    (Health:1;Score:30;Speed:1;Vector:-1;TimeReload:0;RadiusAgr:150;
     Jump:false;HeightJump:0),
    (Health:2;Score:50;Speed:0;Vector:-1;TimeReload:0;RadiusAgr:700;
     Jump:false;HeightJump:0),
    (Health:1;Score:80;Speed:2;Vector:-1;TimeReload:0;RadiusAgr:300;
     Jump:false;HeightJump:0)
    );

type TEnemy = class
       Rect      : TRect;
       Frame     : real;
       ID        : integer;
       Active    : boolean;
       Health    : integer;
       Score     : integer;
       Speed     : integer;
       Vector    : integer;
       TimeReload: integer;
       RadiusAgr : integer;
       Life      : boolean;
       Jump      : boolean;
       HeightJump: integer;
       SpeedDown : real;
       Impuls    : real;
       procedure Init(x,y,w,h: integer; idEnemy: byte);
       procedure Process(flagSelect: integer; Redo: boolean);
       procedure Draw(CameraX: integer);
     end;

var texEnemy       : array[1..EnemyMaxCount] of cardinal;
    EnemyTreeActive: boolean;

implementation

uses main,player, level;

{ TEnemy }

procedure TEnemy.Draw(CameraX: integer);
begin
  if Life = false then
    exit;
  DrawSprite2D(texEnemy[ID],Rect.x-CameraX,Rect.y,Rect.w,Rect.h,0,255,$FFFFFF,true,Rect.w,Rect.h,round(Frame),EnemyCount);
  if ID = EnemyTree then
  begin
    case round(Frame) of
      1:DrawSprite2D_simple(texSnowBall,Rect.x-CameraX+43,Rect.y+28,12,12);
      2:DrawSprite2D_simple(texSnowBall,Rect.x-CameraX+41,Rect.y+26,12,12);
      3:DrawSprite2D_simple(texSnowBall,Rect.x-CameraX+37,Rect.y+8,12,12);
      5:DrawSprite2D_simple(texSnowBall,Rect.x-CameraX+5,Rect.y+28,12,12);
      6:DrawSprite2D_simple(texSnowBall,Rect.x-CameraX+7,Rect.y+26,12,12);
      7:DrawSprite2D_simple(texSnowBall,Rect.x-CameraX+11,Rect.y+8,12,12);
    end;
  end;
  if Active = true then
  begin
    DrawLine2D(Rect.x-CameraX,Rect.y,Rect.x+Rect.w-CameraX,Rect.y,$0000FF);
    DrawLine2D(Rect.x+Rect.w-CameraX,Rect.y,Rect.x+Rect.w-CameraX,Rect.y+Rect.h,$0000FF);
    DrawLine2D(Rect.x+Rect.w-CameraX,Rect.y+Rect.h,Rect.x-CameraX,Rect.y+Rect.h,$0000FF);
    DrawLine2D(Rect.x-CameraX,Rect.y+Rect.h,Rect.x-CameraX,Rect.y,$0000FF);
  end;
end;

procedure TEnemy.Init(x, y, w, h: integer; idEnemy: byte);
begin
  Rect:= ReturnRect(x,y,w,h);
  ID:= idEnemy;
  Frame:= 0.7;
  Active:= false;
  Life:= true;
  Health:= EnemyParam[idEnemy].Health;
  Score:= EnemyParam[idEnemy].Score;
  Speed:= EnemyParam[idEnemy].Speed;
  Vector:= EnemyParam[idEnemy].Vector;
  TimeReload:= EnemyParam[idEnemy].TimeReload;
  RadiusAgr:= EnemyParam[idEnemy].RadiusAgr;
  Jump:= EnemyParam[idEnemy].Jump;
  HeightJump:= EnemyParam[idEnemy].HeightJump;
  SpeedDown:= 0;
end;

procedure TEnemy.Process(flagSelect: integer; Redo: boolean);
var numGround  : integer;
    Left,Rigth,
    Up,Down,
    LeftDown,
    RigthDown  : boolean;
    numSnowBall: integer;
begin
  if Life = false then
    exit;
  if Redo = false then
  begin
    Left:= true;
    Rigth:= true;
    Up:= true;
    Down:= true;
    LeftDown:= true;
    RigthDown:= true;
    for numGround:= 0 to Levels[CurrentLevel].countGround-1 do
    begin
      if Levels[CurrentLevel].Ground[numGround].Fon = true then
        continue;
      if RectInRect(Levels[CurrentLevel].Ground[numGround].Rect,
                    ReturnRect(Rect.x+TmpSpeed,Rect.y-TmpSpeed,Rect.w-TmpSpeed*2,Rect.h)) then
        Up:= false;
      if RectInRect(Levels[CurrentLevel].Ground[numGround].Rect,
                    ReturnRect(Rect.x+TmpSpeed,Rect.y+TmpSpeed,Rect.w-TmpSpeed*2,Rect.h)) then
      begin
        Down:= false;
        SpeedDown:= 0;
        Rect.y:= Levels[CurrentLevel].Ground[numGround].Rect.y-Rect.h;
      end;
      if RectInRect(Levels[CurrentLevel].Ground[numGround].Rect,
                    ReturnRect(Rect.x-TmpSpeed,Rect.y,Rect.w,Rect.h)) then
        Left:= false;
      if RectInRect(Levels[CurrentLevel].Ground[numGround].Rect,
                    ReturnRect(Rect.x+TmpSpeed,Rect.y,Rect.w,Rect.h)) then
        Rigth:= false;
      if RectInRect(Levels[CurrentLevel].Ground[numGround].Rect,
                    ReturnRect(Rect.x-Rect.w,Rect.y+TmpSpeed,Rect.w,Rect.h)) then
        LeftDown:= false;
      if RectInRect(Levels[CurrentLevel].Ground[numGround].Rect,
                    ReturnRect(Rect.x+Rect.w*2,Rect.y+TmpSpeed,Rect.w,Rect.h)) then
        RigthDown:= false;
    end;
    case ID of
      EnemyWind:
      begin
        if Down then
          Rect.y:= Rect.y+Round(SpeedDown);
        if Vector = leftVector then
        begin
          if Frame > EnemyCount div 2 then
            Frame:= 0.7
          else
            Frame:= Frame+FrameSpeed;
          if (Left = true) and (LeftDown = false) then
            Rect.x:= Rect.x+Speed*Vector
          else
          begin
            Vector:= rigthVector;
            Frame:= (EnemyCount div 2)-0.3
          end;
        end
        else if Vector = rigthVector then
        begin
          if Frame > EnemyCount then
            Frame:= (EnemyCount div 2)-0.3
          else
            Frame:= Frame+FrameSpeed;
          if (Rigth = true) and (RigthDown = false) then
            Rect.x:= Rect.x+Speed*Vector
          else
          begin
            Vector:= leftVector;
            Frame:= 0.7
          end;
        end;
      end;
      EnemyTree:
      begin
        EnemyTreeActive:= false;
        if Down then
          Rect.y:= Rect.y+Round(SpeedDown);
        if (Rect.x-RadiusAgr < Players.Rect.x) and (Rect.x > Players.Rect.x) then
        begin
          EnemyTreeActive:= true;
          Vector:= leftVector;
          if TimeReload = 0 then
          begin
            if Frame > 4 then
              Frame:= 1
            else
              Frame:= Frame+FrameSpeed;
            if Frame > 4 then
            begin
              TimeReload:= EnemyMaxReload;
              SnowBallCreate(Rect.x-SnowBallWH-RangeSnowBall,Rect.y+(Rect.h div 2)-SnowBallWH,SnowBallWH,SnowBallWH,Vector);
              Frame:= 1;
            end;
          end;
        end;
        if (Rect.x+RadiusAgr > Players.Rect.x) and (Rect.x < Players.Rect.x) then
        begin
          EnemyTreeActive:= true;
          Vector:= rigthVector;
          if TimeReload = 0 then
          begin
            if Frame < 5 then
              Frame:= 5
            else
              Frame:= Frame+FrameSpeed;
            if Frame > 8 then
            begin
              TimeReload:= EnemyMaxReload;
              SnowBallCreate(Rect.x+Rect.w+RangeSnowBall,Rect.y+(Rect.h div 2)-SnowBallWH,SnowBallWH,SnowBallWH,Vector);
              Frame:= 5;
            end;
          end;
        end;
        if EnemyTreeActive = false then
        begin
          if Frame > 2.3 then
            Frame:= 1
          else
            Frame:= Frame+FrameSpeed;
        end;
      end;
      EnemySold:
      begin
        if (Down = true) and (Jump = false) then
        begin
          Rect.y:= Rect.y+Round(SpeedDown);
          if SpeedDown < SpeedDownMax then
            SpeedDown:= SpeedDown+TestSilaTyajesty;
        end;
        if Jump = false then
        begin
          if SpeedDown > 0 then
          begin
            if Vector = leftVector then
              Frame:= 4;
            if Vector = rigthVector then
              Frame:= 8;
          end;
          if Vector = leftVector then
          begin
            if (Left = true) and (LeftDown = false) and (SpeedDown = 0) then
            begin
              Rect.x:= Rect.x+Speed*Vector;
              if Frame > 3 then
                Frame:= 0.7
              else
                Frame:= Frame+FrameSpeed;
            end
            else if SpeedDown = 0 then
            begin
              Vector:= rigthVector;
              Frame:= 4.7;
            end;
          end
          else if Vector = rigthVector then
          begin
            if (Rigth = true) and (RigthDown = false) and (SpeedDown = 0) then
            begin
              Rect.x:= Rect.x+Speed*Vector;
              if Frame > 7 then
                Frame:= 4.7
              else
                Frame:= Frame+FrameSpeed;
            end
            else
            begin if SpeedDown = 0 then
              Vector:= leftVector;
              Frame:= 0.7;
            end;
          end;
          if SpeedDown = 0 then
          begin
            for numSnowBall:= 0 to Length(SnowBalls)-1 do
            begin
              if SnowBalls[numSnowBall].active = true then
              begin
                if (Rect.y <= SnowBalls[numSnowBall].rect.y+SnowBalls[numSnowBall].rect.h) and
                   (Rect.y+Rect.h >= SnowBalls[numSnowBall].rect.y)
                   and
                   (
                    ((Rect.x-WarningDist < SnowBalls[numSnowBall].rect.x+SnowBalls[numSnowBall].rect.w)
                      and (SnowBalls[numSnowBall].vector = leftVector)) or
                    ((Rect.x+Rect.w+WarningDist > SnowBalls[numSnowBall].rect.x)
                      and (SnowBalls[numSnowBall].vector = rigthVector))
                   ) then
                begin
                  Jump:= true;
                  Impuls:= TestBeginImpuls;
                  SpeedDown:= 0;
                  Break;
                end;
              end;
            end;
          end;
        end
        else
        begin
          if Vector = leftVector then
            Frame:= 4;
          if Vector = rigthVector then
            Frame:= 8;
          Impuls:= Impuls-TestSilaTyajesty;
          if Up then
            Rect.y:= Rect.y-round(impuls)
          else
          begin
            Jump:= false;
            SpeedDown:= 0.1;
          end;
          if Impuls <= 0 then
          begin
            Jump:= false;
            SpeedDown:= 0.1;
          end;
        end;
      end;
    end;
    if (RectInRect(Rect,Players.Rect) = true) and (Players.TimeNoKill = 0) then
      Players.IncHealth(-1);
    if TimeReload < 0 then
      TimeReload:= 0
    else if TimeReload > 0 then
      TimeReload:= TimeReload-1;
    if (Health <= 0) or (Rect.y > wnd.Height) then
    begin
      Life:= false;
      EffectCreate(Rect.x, Rect.y, Rect.w, Rect.h,false);
      ScoreCreate(Rect.x, Rect.y, Score);
    end;
  end;
  if flagSelect <> 0 then
    exit;
  if PointInRect(CursorX+CameraMainX,CursorY,Rect) then
  begin
    if (GetMouseButtonPressed = MB_LEFT) and (MouseDown = MB_NONE) then
    begin
      if Active = true then
        Active:= false
      else
        Active:= true;
    end;
  end;
end;

end.
