unit uCamera;

interface
uses uBB, uUtils, OpenGL, sNail;
const
 SCREEN_WIDTH = 1024;
 SCREEN_HEIGHT = 768;

type
  TCamera = object
  private
    target: PVector;
    BB: cpBB;
  public
    pos: TVector;
    sWidth, sHeight: single;
    MaxWidth, MaxHeight: cardinal;
    z: single;

    procedure SetDefault;
    procedure SetTarget(p: PVector);
    procedure Move;
    procedure Scale;
    procedure Update;
    function WithinBB(box: cpBB): boolean;
  end;

var
  camera: TCamera;
  
implementation

{ TCamera }

procedure TCamera.Move;
begin
  Scale;
  glTranslatef(-pos.x, -pos.y, 0);
end;

procedure TCamera.Scale;
begin
  glScalef(sWidth * z, sHeight * z, 1);
end;

procedure TCamera.SetDefault;
begin
  z := 1.0;
  sWidth := gl_GetResX / SCREEN_WIDTH;
  sHeight := gl_GetResY / SCREEN_HEIGHT;
  target := @veczero;
  pos := veczero;
end;

procedure TCamera.SetTarget(p: PVector);
begin
  target := p;
end;

procedure TCamera.Update;
var
  w, h: word;
begin
  pos.x := 0;
  pos.y := 0;
  w := SCREEN_WIDTH shr 1;
  h := SCREEN_HEIGHT shr 1;

  if MaxWidth > SCREEN_WIDTH then
  begin
    if target.x > w then pos.x := target.x - w;
    if target.x + w > (MaxWidth) then pos.x := (MaxWidth) - SCREEN_WIDTH;
  end;

  if MaxHeight > SCREEN_HEIGHT then
  begin
    if target.y > h then pos.y := target.y - h;
    if target.y + h > (MaxHeight) then pos.y := (MaxHeight) - SCREEN_HEIGHT;
  end;

  BB := cpBBNew(pos.x, SCREEN_HEIGHT + pos.y, SCREEN_WIDTH + pos.x, pos.y);
end;

function TCamera.WithinBB(box: cpBB): boolean;
begin
  if z <>  1.0 then
  begin
    Result := true;
    exit;
  end;
  Result := cpBBinBB(bb, box);
end;

initialization


end.
 