unit uGame;

interface
uses sNail, NFAFont, OpenGL, Windows,uParticles, uLevel, uPlayer, uUtils, uSpace,
     uBody, uShape, uVect, uArbiter, uCollision, uCamera, uJoint;
type
  TGame = object
  private
    level: TLevel;
    levelNum:byte;
    minutes, seconds: single;
    timeStr: string;
    showInf: boolean;

    procedure UpdateTime;
    procedure RenderTime;
    procedure SetTime(const sec: word);
    procedure ShowInfo;
  public
    player:TPlayer;
    procedure Init;
    procedure NextLevel;
    procedure Collision;
    procedure Reset;
    procedure Update;
    procedure Render;
    procedure Free;
  end;

  procedure SndPlay(id: integer);
var
  fntMain, fntNum: TNFAFont;
  texHud: cardinal;
  sndFinish, sndCrash, sndJump,
  sndBonus, sndEat, sndDrink: integer;
  game:TGame;
  gameStarted: boolean = false;

const
  LEVEL_COUNT = 8;
  
implementation
uses uMenu;

procedure SndPlay(id: integer);
begin
  if cfg.sEnable then snd_Play(id);
end;

{ TGame }
procedure TGame.Init;
begin
  cpSpaceInit;
  space.gravity := cpv(0, 100);

  levelNum := 1;
  level.Init(@player);
  level.Load(LevelNum);
  player.Init(level.SPoint);
  camera.SetDefault;
  camera.SetTarget(@player.pos);
  SetTime(level.TimeAll);
  showInf := false;
  Update;
end;

procedure TGame.Reset;
begin
  level.ResetMe;
  player.Reset(Level.SPoint);
  SetTime(level.TimeAll);
end;

procedure TGame.Update;
var
  i, steps: integer;
  dt: single;
begin
  //if inp_KeyUp(VK_F1) then NextLevel;
  if not gui.Active then
  begin
    if (not showInf) and gameStarted then ShowInfo;
    if inp_KeyUp(VK_RETURN) then Reset;
    if inp_KeyUp(VK_ESCAPE) then gui.Activate('MainMenu');

    if not level.PlayerInLevel then
    begin
      SndPlay(sndCrash);
      gui.Activate('Crash');
    end;

    if abs(VectDist(player.pos, level.FPoint)) < 40 then
    begin
      SndPlay(sndFinish);
      if level.Presents = level.PresentsMax then
      begin
        if levelNum = LEVEL_COUNT then
        begin
          gameStarted := false;
          levelNum := 0;
          NextLevel;
          gui.Activate('GameOver')
        end
        else
          gui.Activate('FinishWin');
      end else
        gui.Activate('FinishLose');
    end;

    UpdateTime;
    steps := 3;
	  dt := 1 / 60 / steps;
	  for i:= 0 to steps - 1 do
    begin
      player.Update;
      level.Update(player.pos);
      Collision;
		  cpSpaceStep(dt);
    end;
    camera.Update;
  
    level.PostUpdate;
    parts.Update;
  end;
end;

procedure TGame.Render;
begin
  level.BeforeRender;

  camera.Move;
  glPushMatrix;
    level.Render;
    parts.Render;
    player.Render;
  glPopMatrix;

  glLoadIdentity;
  level.AfterRender;
  camera.Scale;

  if gameStarted then
  begin
    RenderTime;
    Level.RenderHUD;

    gl_SetColor($ffffff, 210);
    gl_TexEnable(texHud);
    glPushMatrix;
      gl_RenderTex(0, 50, 256, 128 , 0);
    glPopMatrix;
    gl_TexDisable;

    gl_SetColor($ffffff);
    fntMain.Render(10, 60, level.Title);
    gl_SetColor($333333);
    fntMain.Render(35, 100, Int2Str(player.hop));
  end;
end;

procedure TGame.Free;
begin
  parts.Free;
  player.Free;
  cpSpaceClear;
end;

procedure TGame.NextLevel;
begin
  cpSpaceClear;
  parts.Clear;
  Inc(levelNum);
  Level.Load(levelNum);
  Player.Init(level.SPoint);
  SetTime(level.TimeAll);
  showInf := false;
end;

procedure TGame.Collision;
var
  s: PShape;
begin
  s := cpShape;
  while s <> nil do
  begin
    player.Collide(s);
    s := s.Next;
  end;
end;

procedure TGame.UpdateTime;
begin
  seconds := seconds - 0.001 * snail_GetTimer;

  if (Round(minutes) = 0) and (Round(seconds) = 10) then
    player.ret.MakeRand(RETORT_HURRY);

  if Round(seconds) = 0.0 then
  begin
    if Round(minutes) = 0 then
    begin
      SndPlay(sndFinish);
      gui.Activate('FinishLose');
    end else
    begin
      minutes := minutes - 1;
      seconds := 59;
    end;
  end;

  timeStr := Numb2Str(Round(minutes)) + ':' + Numb2Str(Round(seconds));
end;

procedure TGame.SetTime(const sec: word);
begin
  minutes := sec div 60;
  seconds := sec mod 60;
end;

procedure TGame.RenderTime;
begin
  gl_SetColor($ffffff, 230);
  fntNum.Render((SCREEN_WIDTH shr 1) - (fntNum.StrWidth(timeStr) shr 1), 70, timeStr);
end;

procedure TGame.ShowInfo;
begin
  if levelNum <= 5 then
  begin
    case LevelNum of
      1: LvlDescription('!        .'+#13+
                        ':'+#13+'" "  "W" -  '+#13+
                        '" "  "S" -    ()'+#13+
                        '"  "  "A/D" -  ,'+#13+
                        '       .'+#13#13+
                        ':    .');

      2: LvlDescription('    .'+#13+
                        '   ,   "" .'+#13+
                        '       .'+#13+
                        '  ""  .   ,'+#13+
                        '  ""    .'+#13+
                        '  :      .'
                        +#13#13+
                        ':    .');

      3: LvlDescription(',  .     !'+#13+
                        '   ,    ' +#13+
                        '   .   ,'+#13+
                        '    ,    .'+#13#13+
                        ':      .');

      4: LvlDescription('    ,   '+#13+
                        '  .     '+#13+
                        ' ,       '+#13+
                        ' .     ,  ,'+#13+
                        '     ,    '+#13+
                        ' .'+#13#13+
                        ':  .');

      5: LvlDescription(' ,          '+#13+
                        '      -  !' +#13+
                        ' !  !' +#13#13+ 
                        ':  -.');
    end;
    gui.Activate('Info');
  end else
    player.ret.MakeRand(RETORT_CUE);
  showInf := true;
end;

end.
