unit uGui;

interface
uses Windows, OpenGL, sNail, uGame, uCamera;

type
  PGuiFrame = ^TGuiFrame;
  TGuiFrame = class;
  PGuiObj = ^TGuiObj;
  TGuiObj = class
  public
    parent: TGuiFrame;
    name: string;
    x, y: smallint;
    caption: string;
    active, visible: boolean;
    constructor Create(name: string; x, y: smallint; p: TGuiFrame); virtual;
    procedure Update; virtual;
    procedure Render; virtual;
  end;

  TGuiLabel = class(TGuiObj)
  private
    color: integer;
  public
    procedure Render; override;
  end;

  TGuiButton = class(TGuiObj)
  private
    r: sRect;
    onClick: procedure;
    isClick: boolean;
    down: boolean;
  public
    constructor Create(name: string; x, y: smallint; p: TGuiFrame); override;
    procedure Update; override;
    procedure Render; override;
  end;

  TGuiFrame = class(TGuiObj)
  private
    count: integer;
    objs: array of TGuiObj;
    alpha: byte;
  public
    constructor Create(name: string; x, y: smallint; p: TGuiFrame); override;
    procedure Render; override;
    procedure Update; override;
    function Add(obj: TGuiObj): PGuiFrame;
    function AddButton(name: string; x, y: smallint; capt: string; event: pointer): PGuiFrame;
    function AddLabel(name:string; x, y: smallint; text: string; color: integer = $333333): PGuiFrame;
    function Element(n: string): PGuiObj;
  end;

  TGuiMachine = class
  protected
    count: integer;
    objs: array of TGuiFrame;
    curFrame: integer;
    prevFrame: string;
  public
    constructor Create;
    procedure Update;
    procedure Render;
    procedure Add(frame: TGuiFrame);
    function AddFrame(name, capt: string; fillBg: boolean = true): PGuiFrame;
    function GetId(n: string): integer;
    function Frame(n: string): PGuiFrame;
    procedure Activate(n: string);
    procedure ActivePrev;
    procedure Deactivate;
    function Active: boolean;
  end;

var
  texGui, texCursor: cardinal;
  cursorPos: TPoint;
  sndClick: integer;
  
implementation
uses NFAFont;

const
  FRAME_WIDTH = 512;
  FRAME_HEIGHT = 280;
  FRAME_BORDER = 20;
  BTN_WIDTH = 186;
  BTN_HEIGHT = 39;

  FRAME_CENTER_X = (SCREEN_WIDTH shr 1) - (FRAME_WIDTH shr 1);
  FRAME_CENTER_Y = (SCREEN_HEIGHT shr 1) - (FRAME_HEIGHT shr 1);
  BTN_CENTER = (FRAME_WIDTH shr 1) - (BTN_WIDTH shr 1);
{ TGuiObj }

function PinR(p: TPoint; r: sRect): boolean;
begin
  Result := (p.x >= r.x) and (p.Y >= r.y) and (p.x <= r.w) and (p.y <= r.h);
end;

constructor TGuiObj.Create(name: string; x, y: smallint; p: TGuiFrame);
begin
  parent := p;
  self.name := name;

  if p = nil then
  begin
    if x = 0 then self.x := FRAME_CENTER_X else self.x := x;
    if y = 0 then self.y := FRAME_CENTER_Y else self.y := y;
  end else
  begin
    if x <> 0 then self.x := x + parent.x else self.x := parent.x + BTN_CENTER;
    if y <> 0 then self.y := y + parent.y else self.y := parent.y + (parent.count * (BTN_HEIGHT + 5)) + 55;
  end;

  active := true;
  visible := true;
  caption := name;
end;

procedure TGuiObj.Render;
begin
end;

procedure TGuiObj.Update;
begin
end;

{ TGuiLabel }

{procedure AddLinesFeed(var s: string);
var
  i, len: word;
  buf: string;
  t: string;
begin
  t := '';

  for i := 0 to Length(s) do
  begin
    buf := buf + s[i];
    len := fntMain.StrWidth(buf);
    if s[i] = #13 then
    begin
      len := 0;
      buf := '';
    end;
    
    if len >= FRAME_WIDTH - 20 then
    begin
      t := t + #13;
      buf := '';
    end;
    t := t + s[i];
  end;

  s := t;
end;
}

procedure TGuiLabel.Render;
begin
  inherited;
  gl_SetColor(color);
  fntMain.Render(x, y, caption);
end;

{ TGuiButton }

constructor TGuiButton.Create(name: string; x, y: smallint; p: TGuiFrame);
begin
  inherited;
  onClick := nil;
  isClick := false;
  down := false;
  r := makeRect(self.x, self.y, self.x + BTN_WIDTH, self.y + BTN_HEIGHT);
end;

procedure TGuiButton.Update;
begin
  inherited;
  if PinR(cursorPos, r) then
  begin
    down := inp_MousePress(MOUSE_BTN_LEFT);
    if inp_MouseUp(MOUSE_BTN_LEFT) then
    begin
       onClick;
       SndPlay(sndClick);
    end;
  end else down := false;
end;

procedure TGuiButton.Render;
begin
  inherited;
  glColor3f(1, 1, 1);
  gl_TexEnable(texGui);
    glPushMatrix;
    if down then
      gl_RenderClip(texGui, x, y, BTN_WIDTH, BTN_HEIGHT, makeRect(185, 282, BTN_WIDTH, BTN_HEIGHT), 0)
    else
      gl_RenderClip(texGui, x, y, BTN_WIDTH, BTN_HEIGHT, makeRect(0, 282, BTN_WIDTH, BTN_HEIGHT), 0);

    glPopMatrix;
  gl_TexDisable;
  fntMain.Render((x) - (fntMain.StrWidth(caption) / 2) + (BTN_WIDTH / 2), y + 8, caption);
end;

{ TGuiFrame }

function TGuiFrame.Add(obj: TGuiObj): PGUIFrame;
begin
  SetLength(objs, Length(objs) + 1);
  objs[count] := obj;
  inc(count);
  Result := @Self;
end;

function TGuiFrame.AddButton(name: string; x, y: smallint; capt: string;
  event: pointer): PGuiFrame;
var
  btn: TGuiButton;
begin
  btn := TGuiButton.Create(name, x, y, self);
  with btn do
  begin
    caption := capt;
    onClick := event;
  end;
  Result := Add(btn);
end;

function TGuiFrame.AddLabel(name:string; x, y: smallint; text: string; color: integer = $333333): PGuiFrame;
var
  lbl: TGuiLabel;
begin
  //AddLinesFeed(text);
  lbl := TGuiLabel.Create(name, x, y, self);
  lbl.caption := text;
  lbl.color := color;
  Result := Add(lbl);
end;

constructor TGuiFrame.Create(name: string; x, y: smallint; p: TGuiFrame);
begin
  inherited;
  objs := nil;
  count := 0;
end;

function TGuiFrame.Element(n: string): PGuiObj;
var
  i: word;
begin
  Result := nil;
  for i := 0 to count - 1 do
    if objs[i].name = n then
    begin
      Result := @objs[i];
      break;
    end;
end;

procedure TGuiFrame.Render;
var
  i: word;
begin
  inherited;
  if not visible then exit;

  gl_SetColor($000000, alpha);
  glPushMatrix;
    glRect(0, 0, SCREEN_WIDTH, SCREEN_HEIGHT);
  glPopMatrix;

  glColor4f(1, 1, 1, 1);
  gl_TexEnable(texGui);
  glPushMatrix;
    gl_RenderClip(texGui, x, y, FRAME_WIDTH, FRAME_HEIGHT, makeRect(0, 0, FRAME_WIDTH, FRAME_HEIGHT), 0);
  glPopMatrix;
  gl_TexDisable;

  glColor3f(0, 0, 0);
  fntMain.Render(x + (FRAME_WIDTH / 2) - (fntMain.StrWidth(caption) / 2), y + 10, caption);

  if count > 0 then
    for i := 0 to count - 1 do
      if objs[i].visible then objs[i].Render;

  glColor4f(1, 1, 1, 0.8);
  gl_TexEnable(texCursor);
    glPushMatrix;
      gl_RenderTex(cursorPos.X, cursorPos.Y, 16, 16, 0);
    glPopMatrix;
  gl_TexDisable;
end;

procedure TGuiFrame.Update;
var
  i: word;
begin
  inherited;
  if (not active) or (not visible) then exit;
  
  if count > 0 then
    for i := 0 to count - 1 do
      if (objs[i].active) and (objs[i].visible) then objs[i].Update;

  cursorPos := inp_GetMousePos;
end;

{ TGuiMachine }

procedure TGuiMachine.Activate(n: string);
begin
  if curFrame <> -1 then prevFrame := objs[curFrame].name;
  curFrame := GetId(n);
end;

procedure TGuiMachine.Add(frame: TGuiFrame);
begin
  SetLength(objs, Length(objs) + 1);
  objs[count] := frame;
  inc(count);
end;

function TGuiMachine.AddFrame(name, capt: string; fillBg: boolean = true): PGuiFrame;
var
  frame: TGuiFrame;
begin
  frame := TGuiFrame.Create(name, 0, 0, nil);
  frame.caption := capt;
  if fillBg then
    frame.alpha := 200
  else
    frame.alpha := 0;
  Add(frame);
  Result := @frame;
end;

constructor TGuiMachine.Create;
begin
  objs := nil;
  count := 0;
  curFrame := -1;
  prevFrame := '';
end;

procedure TGuiMachine.Deactivate;
begin
  prevFrame := objs[curFrame].name;
  curFrame := -1;
end;

function TGuiMachine.GetId(n: string): integer;
var
  i: integer;
begin
  Result := -1;
  for i := 0 to count - 1 do
    if objs[i].name = n then
    begin
      Result := i;
      break;
    end;
end;

function TGuiMachine.Frame(n: string): PGuiFrame;
var
  id: integer;
begin
  id := GetId(n);
  if id > - 1 then
    Result := @objs[id]
  else
    Result := nil
end;

procedure TGuiMachine.Render;
begin
  if curFrame <> -1 then
      objs[curFrame].Render;
end;

procedure TGuiMachine.Update;
begin
  if curFrame <> -1 then
      objs[curFrame].Update;
end;

procedure TGuiMachine.ActivePrev;
begin
  if prevFrame <> '' then
    Activate(prevFrame);
end;

function TGuiMachine.Active: boolean;
begin
  Result := (curFrame <> - 1);
end;

end.
 