unit uLevel;

interface
uses sNail, OpenGL, Windows, Math, uPlayer, uUtils, uSpace, uVect, uBody, uShape,
     uBB, uParticles, uPolyShape, uCamera;

type
  TPoly = record
    VCount: byte;
    Collide:boolean;
    TexNum: integer;
    Vertexes : array of TVector;
  end;

  TDecor = record
    x, y : single;
    Alpha : byte;
    Angle : smallint;
    TexNum : integer;
  end;

  TBonus = record
    pos: TVector;
    target: byte;
  end;

  TPhysInf = record
    model: byte;
    pos: TVector;
  end;

  TPhysObj = class
    pos: TVector;
    body: PBody;
    shape: PShape;

    constructor Create(p: TVector); virtual;
    procedure MakeBox(m, u ,w, h: single);
    procedure MakeCircle(m, u , r: single);
    procedure Reset; virtual;
    procedure Render; virtual;
  end;

  TPhysIce = class(TPhysObj)
    constructor Create(p: TVector); override;
    procedure Render; override;
  end;

  TPhysLog = class(TPhysObj)
    constructor Create(p: TVector); override;
    procedure Render; override;
  end;

  TPhysLog2 = class(TPhysObj)
    constructor Create(p: TVector); override;
    procedure Render; override;
  end;

  TBonObj = object
    active: boolean;
    inf: TBonus;
    angle: smallint;
    bb: cpBB;
    procedure Render;
  end;

  TLevel = object
  private
    BgPosX : single;
    BgX: single;
    PresPos: word;
    Polygons: array of TPoly;
    Decors: array of TDecor;
    Bonuses: array of TBonObj;
    PhysObjs: array of TPhysObj;
    PolyBB: array of cpBB;
    DecorBB: array of cpBB;
    snowflakes: TSnowFlakes;
  public
    Title: string;
    TimeAll: word;
    PlaPtr: PPlayer;
    SPoint, FPoint: TVector;
    PCount, DCount, BCount: integer;
    PolyList: word;

    Presents: word;
    PresentsMax: word;
    procedure Init(player: PPlayer);
    procedure Clear;
    procedure Load(Num: word);
    procedure Render;
    procedure RenderHUD;
    procedure BeforeRender;
    procedure AfterRender;
    procedure Update(p: TVector);
    procedure ResetMe;
    procedure PostUpdate;
    procedure AddPoly(Poly: TPoly);
    procedure AddPolyBB(id: word);
    function PlayerInLevel: boolean;
  end;

  procedure LoadTexGrounds(s: string);
  procedure LoadTexDecors(s: string);

var
  texGround: array of cardinal;
  texDecor: array of cardinal;
  texBGround, texBonuses, texBonus, texIce, texLog, texLog2: cardinal;
  tObj: GLUtesselator;

const
  MUSIC_PATH = 'data\music\';
  SOUNDS_PATH = 'data\sounds\';
  TEXTURES_PATH = 'data\textures\';
  GROUNDS_PATH = TEXTURES_PATH + 'grounds\';
  DECORS_PATH = TEXTURES_PATH + 'decors\';

implementation
uses uGame;

procedure LoadTexGrounds(s: string);
begin
  SetLength(texGround, Length(texGround) + 1);
  texGround[High(texGround)] := tex_Load(nil, PChar(GROUNDS_PATH + s), false);
end;

procedure LoadTexDecors(s: string);
begin
  SetLength(texDecor, Length(texDecor) + 1);
  texDecor[High(texDecor)] := tex_Load(nil, PChar(DECORS_PATH + s), true);
end;

procedure TLevel.AddPolyBB(id: word);
var
  i: word;
  box: cpBB;
begin
  box := cpBBNew(High(integer), 0.0 , 0.0, High(integer));
  
  with Polygons[id], box do
    for i := 0 to VCount - 1 do
    begin
      l := Min(l, Vertexes[i].x);
      t := Min(t, Vertexes[i].y);
      r := Max(r, Vertexes[i].x);
      b := Max(b, Vertexes[i].y);
    end;
    
  PolyBB[id] := box;
end;

{ TLevel }
procedure TLevel.AddPoly(Poly: TPoly);
var
  i: word;
  sbody: PBody;
  shape: PShape;
  v1, v2: cpVect;
type
  TVect = array [0..2] of double;
  TArray4D = array [0..3] of double;
  PTArray4D = ^TArray4D;
  TArray4F = array [0..3] of single;
  PTVect = ^TVect;

  procedure vertexCallBack(vertex: pointer); stdcall;
  var
    v: TVector;
  begin
    v := TVector(vertex^);
    glTexCoord2f(v.x / 256, v.y / 256);
    glVertex2dv(vertex);
  end;

  procedure combineCallback(coords : PTVect; data : PTArray4D;
            weight : TArray4F; var dataout : Pointer); stdcall;
  var
    vertex : PTVect;
  begin
    GetMem (vertex, SizeOf (TVector));
    vertex^[0] := coords[0];
    vertex^[1] := coords[1];
    vertex^[2] := coords[2];

    dataOut := vertex;
  end;
begin
  gluTessCallback(tobj, GLU_TESS_VERTEX, @vertexCallback);
  gluTessCallback(tobj, GLU_TESS_BEGIN, @glBegin);
  gluTessCallback(tobj, GLU_TESS_END, @glEnd);
  gluTessCallback(tobj, GLU_TESS_COMBINE, @combineCallback);
  gluTessProperty(tobj, GLU_TESS_WINDING_RULE, GLU_TESS_WINDING_POSITIVE);

  glNewList(PolyList, GL_COMPILE);
    gl_TexEnable(texGround[Poly.TexNum]);
    glPushMatrix;
    glColor3f(1, 1, 1);

    gluTessBeginPolygon(tobj, nil);
    gluTessBeginContour(tobj);

    with Poly do
       for i:=0 to VCount - 2 do
         gluTessVertex(tobj, @Vertexes[i], @Vertexes[i]);

    gluTessEndContour(tobj);
    gluTessEndPolygon(tobj);
    glPopMatrix;
    gl_TexDisable;

    gl_SetColor($3f3f3f);
    glPushMatrix;
      glBegin(GL_LINES);
      with Poly do
        for i:=0 to VCount - 2 do
        begin
          glVertex2dv(@Vertexes[i]);
          glVertex2dv(@Vertexes[i + 1]);
        end;  
      glEnd;
    glPopMatrix;
  glEndList;
  Inc(PolyList);

  with Poly do
  begin
    if Collide then
    begin
      sbody := cpBodyCreate(Infinity, Infinity);
      for i:=0 to VCount - 2 do
      begin
        v1 := Vect2cpv(Vertexes[i]);
        v2 := Vect2cpv(Vertexes[i + 1]);
        shape := cpSegmentShapeCreate(sbody, v1, v2, 0);
        shape.u := 1.0;
        shape.e := 0.15;
      end;
    end;

    for i:=0 to VCount - 1 do
    begin
      with camera do
      begin
        MaxWidth := Max(MaxWidth, Round(vertexes[i].x));
        MaxHeight := Max(MaxHeight, Round(vertexes[i].y));
      end;
    end;
  end;
end;

procedure TLevel.Clear;
begin
  SPoint := SeTVector(0, 0);
  FPoint := SeTVector(0, 0);

  SetLength(Polygons, 0);
  SetLength(PolyBB, 0);
  PCount := 0;

  SetLength(Decors, 0);
  DCount := 0;

  glDeleteLists(1, PolyList);
  PolyList := 1;

  Presents := 0;
  PresentsMax := 0;
  snowflakes.Init;
end;

procedure TLevel.Init(player: PPlayer);
begin
  PlaPtr := player;
end;

procedure TLevel.Load(num: word);
var
  f: file of byte;
  i: word;
  s: TPoint;
  pocount: integer;
  physInf: TPhysInf;
  len: byte;
begin
  Clear;

  AssignFile(f, PChar('data\levels\level'+Int2Str(num)+'.rlv'));
  Reset(f);
  BlockRead(f, len, sizeof(len));
  SetLength(Title, len);
  BlockRead(f, Title[1], len);
  BlockRead(f, TimeAll, sizeof(TimeAll));
  BlockRead(f, SPoint, sizeof(TVector));
  BlockRead(f, FPoint, sizeof(TVector));
  BlockRead(f, PCount, sizeof(PCount));
  
  SetLength(Polygons, PCount+1);
  SetLength(PolyBB, PCount+1);
  for i:=0 to PCount do
  begin
    with Polygons[i] do
    begin
      BlockRead(f, VCount, sizeof(VCount));
      BlockRead(f, Collide, sizeof(Collide));
      BlockRead(f, TexNum, sizeof(TexNum));
      SetLength(Vertexes, VCount+1);
      BlockRead(f, Vertexes[0], sizeof(TVector) * VCount);
    end;
    AddPoly(Polygons[i]);
    AddPolyBB(i);
  end;

  BlockRead(f, DCount, sizeof(DCount));
  SetLength(Decors, DCount + 1);
  SetLength(DecorBB, DCount + 1);
  BlockRead(f, Decors[0], sizeof(TDecor) * DCount);

  for i := 0 to DCount do
    with Decors[i] do
    begin
      s := tex_GetSize(texDecor[TexNum]);
      DecorBB[i] := cpBBNew(x, Decors[i].y + s.y, Decors[i].x + s.x, y);
    end;

  BlockRead(f, BCount, sizeof(BCount));
  SetLength(Bonuses, BCount+1);

  if BCount > 0 then
    for i:=0 to BCount -1 do
      with Bonuses[i] do
      begin
        BlockRead(f, Bonuses[i].inf, sizeof(TBonus));
        active := true;
        with inf.pos do
          bb := cpBBNew(x - 16, y + 16, x + 16, y - 16);
        if inf.target = 0 then inc(PresentsMax);
      end;
  
  BlockRead(f, pocount, sizeof(pocount));
  log_AddStr(nil, PChar('- Physics objects: '+Int2Str(pocount)));

  PhysObjs := nil;
  if pocount > 0 then
  for i:= 0 to pocount - 1 do
  begin
    BlockRead(f, physInf, sizeof(TPhysInf));
    SetLength(PhysObjs, Length(PhysObjs) + 1);
    case physInf.model of
      0: PhysObjs[High(PhysObjs)] := TPhysIce.Create(physInf.pos);
      1: PhysObjs[High(PhysObjs)] := TPhysLog.Create(physInf.pos);
      2: PhysObjs[High(PhysObjs)] := TPhysLog2.Create(physInf.pos);
    end;
  end;

  log_AddStr(nil, PChar('- Level Loaded: #'+Int2Str(Num)));
  CloseFile(f);

  PresPos := (SCREEN_WIDTH shr 1) - ((PresentsMax * 24) shr 1);
  parts.Add(PART_GOLDCIRCLE, @FPoint, 2.0);
end;

procedure TLevel.Update(p: TVector);
var
  i: word;
begin
  if Length(Bonuses) > 0 then
    for i:=0 to BCount - 1 do
      with Bonuses[i] do
      begin
        if active then
        begin
          angle := round(RadToDeg(sin(snail_GetCPUTick / 200)));
          if vectDist(PlaPtr.pos, inf.pos) < 50 then
          begin
            case inf.target of
              0:
              begin
                SndPlay(sndBonus);
                Inc(Presents);
              end;
              1:
              begin
                SndPlay(sndEat);
                Inc(PlaPtr.hop);
              end;
              2:
              begin
                SndPlay(sndDrink);
                PlaPtr.Nitro;
              end;
            end;
              parts.Add(PART_SPARKS, @inf.pos, 1.5);
              active := false;
          end;
        end;
      end;
end;

procedure TLevel.PostUpdate;
begin
  BgX := camera.pos.x / 600 / 8;
  BgPosX := BgX  * 2.5;
  snowflakes.Update(-BgPosX / 4);
end;

procedure TLevel.Render;
var
  i: word;
  s: TPoint;
begin
  if Length(Decors) > 0 then
    for i:=0 to DCount -1  do
      with Decors[i] do
      begin
        if not camera.WithinBB(DecorBB[i]) then continue;
        s := tex_GetSize(texDecor[TexNum]);
        glColor4ub(255,255,255, Alpha);
        gl_TexEnable(texDecor[TexNum]);
        glPushMatrix;
          gl_RenderTex(x,y, s.x, s.Y, Angle);
        glPopMatrix;
        gl_TexDisable;
      end;

  if Length(Polygons) > 0 then
    for i:=0 to PCount do
    begin
      if camera.WithinBB(PolyBB[i]) then
        glCallList(i + 1);
    end;

  if Length(Bonuses) > 0 then
  begin
    for i:=0 to BCount - 1 do
      with Bonuses[i] do
        if camera.WithinBB(bb) and active then
          Render;
  end;

  glColor4f(1, 1, 1, 1);
  if Length(PhysObjs) > 0 then
  begin
    for i := 0 to High(PhysObjs) do
      with PhysObjs[i] do
        if camera.WithinBB(shape.bb) then
          Render;
  end;

end;

procedure TLevel.BeforeRender;
begin
  glColor3f(1, 1, 1);
  gl_TexEnable(texBGround);
  glPushMatrix;
  glBegin(GL_QUADS);
    glTexCoord2f(BgX, 0); glVertex2f(0, 0);
    glTexCoord2f(BgX + 1 /2 , 0); glVertex2f(gl_GetResX,0);
    glTexCoord2f(BgX + 1 / 2, 0.5); glVertex2f(gl_GetResX, gl_GetResY);
    glTexCoord2f(BgX, 0.5); glVertex2f(0, gl_GetResY);
  glEnd;
  glPopMatrix;
   
  gl_TexDisable;
end;

procedure TLevel.AfterRender;
begin
  snowflakes.Render;
  if not cfg.BgFx then exit;
  gl_SetBlend(BLEND_ADD);
  glColor4f(1.0, 1.0, 1.0, 0.3);
  gl_TexEnable(texBGround);
  glPushMatrix;
  glBegin(GL_QUADS);
    glTexCoord2f(BgPosX, 0); glVertex2f(0, 0);
    glTexCoord2f(BgPosX + 1 / 2, 0); glVertex2f(gl_GetResX,0);
    glTexCoord2f(BgPosX + 1 / 2, 0.5); glVertex2f(gl_GetResX, gl_GetResY);
    glTexCoord2f(BgPosX, 0.5); glVertex2f(0, gl_GetResY);
  glEnd;
  glPopMatrix;
  gl_SetBlend(BLEND_SUB);
  gl_TexDisable;
end;

procedure TLevel.RenderHUD;
var
  i: smallint;
begin
  gl_TexEnable(texBonuses);
  glColor4f(1, 1, 1, 1);
  for i := 0 to Presents - 1  do
  begin
    glPushMatrix;
      gl_RenderSprite(PresPos + (i * 24), 120, 24, 24, 2, 2, 0, 0);
    glPopMatrix;
  end;

  glColor4f(1, 1, 1, 0.35);
  for i := 0 to (PresentsMax - Presents) - 1 do
  begin
    glPushMatrix;
      gl_RenderSprite(PresPos + ((Presents + i) * 24), 120, 24, 24, 2, 2, 0, 0);
    glPopMatrix;
  end;
  gl_TexDisable;
end;

procedure TLevel.ResetMe;
var
  i: word;
begin
  Presents := 0;
  for i := 0 to BCount do
    Bonuses[i].active := true;

  if Length(PhysObjs) > 0 then
    for i := 0 to High(PhysObjs) do
      PhysObjs[i].Reset;
end;


function TLevel.PlayerInLevel: boolean;
begin
  Result := (PlaPtr.pos.x >= 0.0) and (PlaPtr.pos.x <= camera.MaxWidth)
             and (PlaPtr.pos.y <= camera.MaxHeight);
end;

{ TBonObj }

procedure TBonObj.Render;
begin
  if not active then exit;

  glColor4f(1, 1, 1, 0.75);
  gl_TexEnable(texBonus);
  glPushMatrix;
    with inf do
    gl_RenderTex(pos.x, pos.y, 32, 32, -angle);
  glPopMatrix;
  gl_TexDisable;

  glColor4f(1, 1, 1, 1);
  gl_TexEnable(texBonuses);
  glPushMatrix;
    with inf do
    gl_RenderSprite(pos.x, pos.y, 32, 32, 2, 2, target, angle div 2);
  glPopMatrix;
  gl_TexDisable;
end;

{ TPhysObj }

constructor TPhysObj.Create(p: TVector);
begin
  pos := p;
end;

procedure TPhysObj.MakeBox(m, u, w, h: single);
var
  verts:array[0..3] of cpVect;
begin
  verts[0] := cpv(-w / 2, -h / 2);
  verts[1] := cpv(-w / 2, h / 2);
  verts[2] := cpv(w / 2, h / 2);
  verts[3] := cpv(w / 2, -h / 2);
  body := cpBodyCreate(m, cpMomentForPoly(m, 4, verts, cpvzero));

  shape := cpPolyShapeCreate(body, 4, verts, cpvzero);
  shape.u := u;
  Reset;
end;

procedure TPhysObj.MakeCircle(m, u, r: single);
begin
  body := cpBodyCreate(m, cpMomentForCircle(m, r, 0, cpvzero));

  shape := cpCircleShapeCreate(body, r, cpvzero);
  shape.u := u;
  Reset;
end;

procedure TPhysObj.Render;
begin
end;

procedure TPhysObj.Reset;
begin
  body.p := Vect2cpv(pos);
  cpBodyResetParams(body);
end;

{ TPhysBox }

constructor TPhysIce.Create(p: TVector);
begin
  inherited;
  MakeBox(0.4, 0.2, 16, 16);
end;

procedure TPhysIce.Render;
begin
  inherited;
  gl_TexEnable(texIce);
  glPushMatrix;
    gl_RenderTex(body.p.x - 8, body.p.y - 8, 16, 16, Round(RadToDeg(body.a)));
  glPopMatrix;
  gl_TexDisable;
end;

{ TPhysLog }

constructor TPhysLog.Create(p: TVector);
begin
  inherited;
  MakeCircle(1.0, 0.7, 8.0);
end;

procedure TPhysLog.Render;
begin
  inherited;
  gl_TexEnable(texLog);
  glPushMatrix;
    gl_RenderTex(body.p.x - 8, body.p.y - 8, 16, 16, Round(RadToDeg(body.a)));
  glPopMatrix;
  gl_TexDisable;
end;

{ TPhysLog2 }

constructor TPhysLog2.Create(p: TVector);
begin
  inherited;
  MakeBox(2.1, 0.6, 32, 16);
end;

procedure TPhysLog2.Render;
begin
  inherited;
  gl_TexEnable(texLog2);
  glPushMatrix;
    gl_RenderTex(body.p.x - 16, body.p.y - 8, 32, 16, Round(RadToDeg(body.a)));
  glPopMatrix;
  gl_TexDisable;
end;

initialization
  tObj := gluNewTess;
finalization
  gluDeleteTess(tObj);
end.
