unit uMenu;

interface
uses uGui, sNail, uGame, uUtils, uPlayer;

procedure CreateFrames;
procedure LvlDescription(text: string);

var
  gui: TGuiMachine;

implementation
var
  tmpCfg: TConfig;

procedure LvlDescription(text: string);
begin
  gui.Frame('Info').Element('lbText').caption := text;
end;

procedure bQuit;
begin
  snail_Stop;
end;

procedure bBack;
begin
  gui.ActivePrev;
end;

procedure bPlay;
begin
  gui.Deactivate;
  gui.Frame('MainMenu').Element('btnPlay').caption := '';
  gameStarted := true;
end;

procedure bMainMenu;
begin
  gui.Activate('MainMenu');
end;

procedure bConfig;
begin
  tmpCfg := cfg;
  gui.Activate('ConfigMenu');
end;

procedure bAbout;
begin
  gui.Activate('AboutMenu');
end;

function GetResStr(n: byte): string;
begin
  Result := Int2Str(RES_X[n - 1]) + 'x' + Int2Str(RES_Y[n - 1]);
end;

function GetModeStr(b: boolean): string;
begin
  if b then Result := ' ' else Result := '';
end;

function GetPostStr(b: boolean): string;
begin
  if b then Result := '' else Result := '';
end;

procedure bGraph;
begin
  with tmpCfg, gui.Frame('GraphMenu')^ do
  begin
    Element('btnResolution').caption := GetResStr(Resolution);
    Element('btnFullscreen').caption := GetModeStr(FullScreen);
    Element('btnPostFx').caption := GetPostStr(BgFx);
  end;
  gui.Activate('GraphMenu');
end;

procedure bChangeRes;
begin
  inc(tmpCfg.Resolution);
  if tmpCfg.Resolution > RES_N then
    tmpCfg.Resolution := 1;

  gui.Frame('GraphMenu').Element('btnResolution').caption := GetResStr(tmpCfg.Resolution);
end;

procedure bChangeFScreen;
begin
  tmpCfg.FullScreen := not tmpCfg.FullScreen;
  gui.Frame('GraphMenu').Element('btnFullscreen').caption := GetModeStr(tmpCfg.FullScreen);
end;

procedure bChangePostFx;
begin
  tmpCfg.BgFx := not tmpCfg.BgFx;
  gui.Frame('GraphMenu').Element('btnPostFx').caption := GetPostStr(tmpCfg.BgFx);
end;

function GetMusicStr(b: boolean): string;
begin
  if b then Result := '' else Result := ' ';
end;

function GetSoundStr(b: boolean): string;
begin
  if b then Result := '' else Result := '';
end;

function GetVolumeStr(v: byte): string;
begin
  Result := Int2Str(v);
end;

procedure bSound;
begin
  with tmpCfg, gui.Frame('SoundMenu')^ do
  begin
    Element('btnMusic').caption := GetMusicStr(Music);
    Element('btnEnable').caption := GetSoundStr(sEnable);
    Element('btnVolume').caption := GetVolumeStr(Volume);
  end;
  gui.Activate('SoundMenu');
end;

procedure bChangeMusic;
begin
  tmpCfg.Music := not tmpCfg.Music;
  gui.Frame('SoundMenu').Element('btnMusic').caption := GetMusicStr(tmpCfg.Music);
end;

procedure bChangeSound;
begin
  tmpCfg.sEnable := not tmpCfg.sEnable;
  gui.Frame('SoundMenu').Element('btnEnable').caption := GetSoundStr(tmpCfg.sEnable);
end;

procedure bChangeVolume;
begin
  inc(tmpCfg.Volume, 10);
  if tmpCfg.Volume > 100 then
    tmpCfg.Volume := 0;

  gui.Frame('SoundMenu').Element('btnVolume').caption := GetVolumeStr(tmpCfg.Volume);
end;

procedure bApplyConfig;
begin
  cfg := tmpCfg;
  WriteConfig(cfg);
  gui.Activate('ApplyMenu');
end;

procedure bResetLevel;
begin
  game.Reset;
  gui.Deactivate;
end;

procedure bNextLevel;
begin
  game.NextLevel;
  gui.Deactivate;
end;

procedure bHide;
begin
  gui.Deactivate;
  game.player.ret.MakeRand(RETORT_CUE);
end;

procedure bGameOver;
begin
  gui.Frame('MainMenu').Element('btnPlay').caption := '';
  bMainMenu;
end;

procedure CreateFrames;
begin
  gui.AddFrame('MainMenu', '').
  AddButton('btnPlay', 0, 0, '', @bPlay).
  AddButton('', 0, 0, '', @bConfig).
  AddButton('', 0, 0, '', @bAbout).
  AddButton('', 0, 0, '', @bQuit);

  gui.AddFrame('ConfigMenu', '').
  AddButton('', 0, 0, '', @bGraph).
  AddButton('', 0, 0, '', @bSound).
  AddButton('', 0, 0, '', @bApplyConfig).
  AddButton('', 0, 0, '', @bMainMenu);

  gui.AddFrame('GraphMenu', '').
  AddButton('btnResolution', 0, 0, '', @bChangeRes).
  AddButton('btnFullscreen', 0, 0, '', @bChangeFScreen).
  AddButton('btnPostFx', 0, 0, '', @bChangepostFx).
  AddButton('', 0, 0, '', @bBack).
  AddLabel('', 20, 60, ':').
  AddLabel('', 20, 105, ':').
  AddLabel('', 20, 150, ' :');

  gui.AddFrame('SoundMenu', '').
  AddButton('btnMusic', 0, 0, '', @bChangeMusic).
  AddButton('btnEnable', 0, 0, '', @bChangeSound).
  AddButton('btnVolume', 0, 0, '100', @bChangeVolume).
  AddButton('', 0, 0, '', @bBack).
  AddLabel('', 20, 60, ':').
  AddLabel('', 20, 105, ' :').
  AddLabel('', 20, 150, ' :');

  gui.AddFrame('ApplyMenu', '').
  AddLabel('', 50, 110, '     .').
  AddButton('', 0, 200, '',@bMainMenu);

  gui.AddFrame('AboutMenu', '').
  AddLabel('', 20, 50, '  igdc.ru,  #53' +#13+
                               ', :   aka Hex[man]' +#13+
                               'E-Mail: hex.man@mail.ru'+#13#13+
                               ':'+#13+'Chipmunk physics engine' +#13+
                               'sNail engine'+#13+
                               ' 2010').
  AddButton('', 0, 220, '', @bBack);

  gui.AddFrame('Crash', '', false).
  AddLabel('', 135, 110, ' ?   .').
  AddButton('', 0, 200, '', @bResetLevel);

  gui.AddFrame('FinishWin', '', false).
  AddLabel('', 125,110, '!   .'+#13).
  AddButton('', 60, 200, '', @bResetLevel).
  AddButton('', 260, 200, '', @bNextLevel);

  gui.AddFrame('FinishLose','', false).
  AddLabel('', 30, 80, '   !'+#13+'       . ').
  AddButton('', 0, 200, '', @bResetLevel);

  gui.AddFrame('Info', '', false).
  AddButton('', 0, 220, 'OK', @bHide).
  AddLabel('lbText', 20, 45, '');

  gui.AddFrame('GameOver', '').
  AddLabel('', 40, 110, '!  .  .').
  AddButton('', 0, 200, '', @bGameOver);

end;

initialization
  gui := TGuiMachine.Create;
finalization
  gui.Free;
end.
