unit uPlayer;

interface
uses sNail, OpenGL, Windows, uUtils, uGL, uBody, uVect, uShape,
     uPolyShape, uJoint, uArbiter, uCollision, Math, uParticles, uCamera;
type
  TCollide = record
    c: integer;
    pos: TVector;
  end;

  TRetort = object
    active: boolean;
    pos: PVector;
    alpha: byte;
    text: string;
    time: single;
    tx: word;

    procedure Init(p: PVector);
    procedure Make(s: string);
    procedure MakeRand(const dictionary:array of string);
    procedure Update;
    procedure Render;
  end;

  PPlayer = ^TPlayer;
  TPlayer = object
  private
    enginePower: boolean;
    resorUp1, resorUp2: boolean;
  public
    handangle: smallint;
    wheel1, wheel2, body: PBody;
    swheel1, swheel2, sbody: PShape;
    cwheel1, cwheel2: TCollide;

    resor1, resor2: PDampedSpring;
    sresor1, sresor2: PPinJoint;
    pos, epipe, wpos1, wpos2: TVector;
    ret: TRetort;

    hop: word;
    accelerate, accTime: single;
    procedure Init(p: TVector);
    procedure Reset(p: TVector);
    function MotorTorque(b: PBody; maxw : single):single;
    procedure Nitro;
    procedure Update;
    procedure Render;
    procedure DrawSpring(b1, b2: PBody; jnt:PDampedSpring);
    procedure DrawJoint(b1, b2: PBody; jnt:PPinJoint);
    procedure Collide(s: PShape);
    procedure CollideWheel(w, s: PShape; var col: TCollide);
    procedure Free;
  end;

var
  texCar, texRetort:cardinal;
  sndRetort, sndMotor:integer;

const
  WHEEL_RADIUS = 8;
  WHEEL_DIAMETR = WHEEL_RADIUS * 2;
  WHEEL_MASS = 1.0;
  WHEEL_OFFSET_X = 20.0;
  WHEEL_OFFSET_Y = 30.0;
  WHEEL_SPRING_SIZE = 19.4;
  WHEEL_SPRING_SIZE_MAX = WHEEL_SPRING_SIZE + 5.2;
  WHEEL_MAX_VELOCITY = 34;

  PLAYER_GROUP = 1;
  RETORT_CUE: array[0..2] of string = ('  ', '!',
                                       '!');
  RETORT_HURRY: array[0..5] of string = ('','!', '!',
                                       '!', '10 !', '!');
implementation
uses uGame;

procedure TPlayer.Reset(p:TVector);
begin
  pos := p;
  body.p := Vect2cpv(pos);
  wheel1.p := cpvadd(body.p, cpv(-WHEEL_OFFSET_X - 2, WHEEL_OFFSET_Y + WHEEL_RADIUS));
  wheel2.p := cpvadd(Vect2cpv(pos), cpv(WHEEL_OFFSET_X, WHEEL_OFFSET_Y + WHEEL_RADIUS));

  cpBodyResetParams(body);
  cpBodyResetParams(wheel1);
  cpBodyResetParams(wheel2);

  cpBodySetAngle(body, 0);
  cpBodySetAngle(wheel1, 0);
  cpBodySetAngle(wheel2, 0);

  cwheel1.c := 0;
  cwheel2.c := 0;

  resorUp1 := false;
  resorUp2 := false;

  hop := 0;
  accelerate := 0;
  accTime := 0;
end;

procedure TPlayer.Init(p:TVector);
var
  verts: array[0..5] of cpVect;
  joint: PJoint;
begin
  verts[0] := cpv(-18,-22);
  verts[1] := cpv(-30, 5);
  verts[2] := cpv(-30, 30);
  verts[3] := cpv( 30, 30);
  verts[4] := cpv( 30, 12);
  verts[5] := cpv( 10, -22);

  body := cpBodyCreate(1.0, cpMomentForPoly(1.0, Length(verts), verts, cpvzero));

  sbody := cpPolyShapeCreate(body, Length(verts), verts, cpvzero);
  sbody.u := 1.0;
  sbody.group := PLAYER_GROUP;

  wheel1 := cpBodyCreate(WHEEL_MASS, cpMomentForCircle(WHEEL_MASS, WHEEL_RADIUS, 0, cpvzero));

  swheel1 := cpCircleShapeCreate(wheel1, WHEEL_RADIUS, cpvzero);
  swheel1.u := 1.0;
  swheel1.group := PLAYER_GROUP;

  wheel2 := cpBodyCreate(WHEEL_MASS, cpMomentForCircle(WHEEL_MASS, WHEEL_RADIUS, 0, cpvzero));
  swheel2 := cpCircleShapeCreate(wheel2, WHEEL_RADIUS, cpvzero);
  swheel2.u := 0.5;
  swheel2.group := PLAYER_GROUP;

  Reset(p);

  joint := cpPinJointCreate(body, wheel1, cpv(0, WHEEL_OFFSET_Y), cpvzero);
  sresor1 := joint.DfJoint;

  joint := cpPinJointCreate(body, wheel2, cpv(0, WHEEL_OFFSET_Y), cpvzero);
  sresor2 := joint.DfJoint;

  joint := cpDampedSpringCreate(body, wheel1, cpv(-WHEEL_OFFSET_X - 2, WHEEL_OFFSET_Y - 10), cpvzero, WHEEL_SPRING_SIZE, 120, 2.0{270, 7.5});
  resor1 := joint.DfJoint;
  joint := cpDampedSpringCreate(body, wheel2, cpv(WHEEL_OFFSET_X, WHEEL_OFFSET_Y - 10), cpvzero, WHEEL_SPRING_SIZE, 120, 2.0);
  resor2 := joint.DfJoint;

  ret.Init(@pos);
  parts.Add(PART_SMOKE, @epipe, 0);
end;

procedure TPlayer.Update;
begin
  pos := cpv2Vect(body.p);

  cpBodyResetForce(wheel1);
  cpBodyResetForce(wheel2);
  cpBodyResetForce(body);

  if inp_KeyPress(VK_RIGHT) or inp_KeyPress(ord('D')) then
  begin
    enginePower := true;
    resorUp1 := true;

    if resor1.restLength < WHEEL_SPRING_SIZE_MAX then resor1.restLength := resor1.restLength + 1.5;
    if body.w < 1.5 then
      body.t := body.t - MotorTorque(body, 13);
  end;

  if inp_KeyPress(VK_LEFT) or inp_KeyPress(ord('A')) then
  begin
    enginePower := true;
    resorUp2 := true;
    if resor2.restLength < WHEEL_SPRING_SIZE_MAX then resor2.restLength := resor2.restLength + 1.5;
    if body.w > -1.5 then
      body.t := body.t + MotorTorque(body, 13);
  end;

  if inp_KeyUp(VK_LEFT) or inp_KeyUp(ord('A')) then
  begin
    resorUp2 := false;
    body.t := 0;
    body.w := 0;
  end;

  if inp_KeyUp(VK_RIGHT) or inp_KeyUp(ord('D')) then
  begin
    resorUp1 := false;
    body.t := 0;
    body.w := 0;
  end;

  if inp_KeyUp(VK_UP) or inp_KeyUp(VK_DOWN) or inp_KeyUp(ord('W')) or inp_KeyUp(ord('S')) then
  begin
    enginePower := false;
    snd_Stop(sndMotor);
  end;

  if inp_KeyPress(VK_DOWN) or inp_KeyPress(ord('S')) then
  begin
    enginePower := true;
    wheel1.t := wheel1.t + MotorTorque(wheel1, -(WHEEL_MAX_VELOCITY + accelerate));
    wheel2.t := wheel2.t + MotorTorque(wheel2, -(WHEEL_MAX_VELOCITY + accelerate));
  end;

  if inp_KeyPress(VK_UP) or inp_KeyPress(ord('W')) then
  begin
    enginePower := true;
    wheel1.t := wheel1.t - MotorTorque(wheel1, WHEEL_MAX_VELOCITY + accelerate);
    wheel2.t := wheel2.t - MotorTorque(wheel2, WHEEL_MAX_VELOCITY + accelerate);
  end;

  if (inp_KeyUp(VK_SPACE)) and (hop <> 0) then
  begin
    snd_Play(sndJump);
    cpBodyApplyImpulse(body, cpvmult(cpvforangle(body.a - (PI / 2)), 350), cpv(0, 0));
    parts.Add(PART_GOLDSNOW, @wpos1, 2);
    parts.Add(PART_GOLDSNOW, @wpos2, 2);
    dec(hop);
  end;

  if (not resorUp1) and (resor1.restLength > WHEEL_SPRING_SIZE) then
    resor1.restLength := resor1.restLength - 0.2;

  if (not resorUp2) and (resor2.restLength > WHEEL_SPRING_SIZE) then
    resor2.restLength := resor2.restLength - 0.2;

  if accelerate > 0 then
  begin
    accTime := accTime - 0.012;
    if accTime <= 0.0 then
    begin
      accelerate := 0.0;
      accTime := 0.0;
    end;
  end;

  epipe := cpv2Vect(cpvadd(body.p, cpvrotate(cpv(-WHEEL_OFFSET_X, WHEEL_OFFSET_Y), body.rot)));
  wpos1 := cpv2Vect(wheel1.p);
  wpos2 := cpv2Vect(wheel2.p);
  ret.Update;
end;

procedure TPlayer.DrawSpring(b1, b2: PBody; jnt:PDampedSpring);
var
  v1, v2 : cpVect;
begin
  v1 := cpvadd(b1.p, cpvrotate(jnt.anchr1, b1.rot));
  v2 := cpvadd(b2.p, cpvrotate(jnt.anchr2, b2.rot));
  DrawLine(cpv2Vect(v1),cpv2Vect(v2));
end;

procedure TPlayer.DrawJoint(b1, b2: PBody; jnt: PPinJoint);
var
  v1, v2 : cpVect;
begin
  v1 := cpvadd(b1.p, cpvrotate(jnt.anchr1, b1.rot));
  v2 := cpvadd(b2.p, cpvrotate(jnt.anchr2, b2.rot));
  DrawLine(cpv2Vect(v1),cpv2Vect(v2));
end;

procedure TPlayer.Render;
begin
  glColor3f(0, 0, 0);
  DrawSpring(body, wheel1, resor1);
  DrawSpring(body, wheel2, resor2);
  DrawJoint(body, wheel1, sresor1);
  DrawJoint(body, wheel2, sresor2);

  glColor3f(1, 1, 1);
  gl_TexEnable(texCar);
  glPushMatrix;
    gl_RenderClip(texCar, body.p.x - 37, body.p.y - 42, 75, 84, makeRect(0,0,111,118), Round(RadToDeg(body.a)));
  glPopMatrix;

  glPushMatrix;
    gl_RenderClip(texCar, wheel1.p.x - WHEEL_RADIUS, wheel1.p.y - WHEEL_RADIUS, WHEEL_DIAMETR, WHEEL_DIAMETR, makeRect(0,118,17,17),Round(RadToDeg(wheel1.a)));
  glPopMatrix;

  glPushMatrix;
    gl_RenderClip(texCar, wheel2.p.x - WHEEL_RADIUS, wheel2.p.y - WHEEL_RADIUS, WHEEL_DIAMETR, WHEEL_DIAMETR, makeRect(0,118,17,17),Round(RadToDeg(wheel2.a)));
  glPopMatrix;
  gl_TexDisable;

  ret.Render;
end;

function TPlayer.MotorTorque(b: PBody; maxw: single): single;
begin
  Result := 3200 * Min((b.w - byte(enginepower) * maxw) / maxw, 1.0);
end;

procedure TPlayer.Free;
begin
end;

procedure TPlayer.Collide(s: PShape);
begin
  CollideWheel(swheel1, s, cwheel1);
  CollideWheel(swheel2, s, cwheel2);
end;

procedure TPlayer.CollideWheel(w, s: PShape; var col: TCollide);
var
  cont: cpContacts;
  b: PBody;
begin
  if (s <> w) and (s.group <> PLAYER_GROUP) then
  begin
    if cpCollideShapes(w, s ,cont) > 0 then
    begin
      inc(col.c);
      if col.c > 12 then
      begin
        b := w.Body;
        col.pos := cpv2Vect(cont[0].p);
        if (abs(b.w) > 12 ) then
          if accelerate = 0.0 then
            parts.Add(PART_SNOWFLAKE, @col.pos, b.w / 34.0)
          else
            parts.Add(PART_GOLDSNOW, @col.pos, b.w / 34.0);

        col.c := 0;
      end;
    end;
  end;
end;

procedure TPlayer.Nitro;
begin
  accelerate := 26;
  accTime := 10;
end;

{ TRetort }

procedure TRetort.Init(p: PVector);
begin
  pos := p;
end;

procedure TRetort.Make(s: string);
begin
  if active then exit;
  active := true;
  text := s;
  alpha := 255;
  time := 0.0;
  tx := fntMain.StrWidth(text) div 2;
  SndPlay(sndRetort);
end;

procedure TRetort.MakeRand(const dictionary:array of string);
var
  s: string;
begin
  s := dictionary[Random(High(dictionary) + 1)];
  if s <> '' then Make(s);
end;

procedure TRetort.Render;
begin
  if not active then exit;

  gl_SetColor($ffffff, Alpha);
  gl_TexEnable(texRetort);
  glPushMatrix;
    gl_RenderTex(pos.x - 64, pos.y - 100, 128, 128, 0);
  glPopMatrix;
  gl_TexDisable;

  gl_SetColor($000000, Alpha);
  fntMain.Render(pos.x - tx, pos.y - 75, text);
end;

procedure TRetort.Update;
begin
  if not active then exit;
    time := time + 0.04;

  if time > 20.0 then
    alpha := alpha - 1;

  if alpha = 0 then
    active := false;
end;

end.
